

close all
clear all

%% Nominal model definition

Ki = 2;        % current loop transconductance [A/V]
Kt = 0.071;    % motor torque const [Nm/A]

%   Nominal model definition
Jm  = 0.000027;      %   motor rotor inertia
Jd1 = 1.08e-5;       %   small disk inertia
Jd2 = 0.000149;      %   large disk inertia
J   = Jm+Jd1+Jd2;    %   tot load inertia
b   = 2.4299e-004;   %   viscous damping coeff

%   DAC & Encoder sampling time
Ts = 0.001;

K = Ki*Kt;
H = tf(1,[J b 0])

% Generazione coppia motrice

Tfin = .5;
t = [0:Ts:Tfin];
dc = 0.4; % duty cycle
T = 1;

u = 1/2+square_wave(t,T,dc);

% generazione coppia resistente
omegar = .5;
urmax = 1;

ur = urmax*(1/2+1/2*sin(omegar*t));

u_eff = u - ur;

y = lsim(H,u_eff,t);

% quantizzazione dell'output (encoder)
Nbits = 3;



[yq Delta] = my_quantizer(y,Nbits);

% rumore equivalente
sigman2=Delta^2/12;


%sigman2 = .1; 
%yq = y + sqrt(sigman2)*randn(size(y));


figure(1)
plot(t,u,'linewidth',2)
hold on
plot(t,ur,':r','linewidth',2)
plot(t,u_eff,':g','linewidth',2);
legend('Coppia Comando','Coppia Resistente','Coppia effettiva')

figure(2)
plot(t,y)
hold on
plot(t,yq,':r','linewidth',2)
legend('Posizione albero','Misura encoder')


% construisce modello discretizzato del motore  per il filtro di Kalman

Hd = c2d(ss(H),Ts,'zoh')
Ad = Hd.a;
Bd = Hd.b;
Cd = Hd.c;
Dd = Hd.d;

% Modello per il disturbo di coppia
% passeggiata aleatoria integrata

Ac = [1 Ts; 0 1];
Bc = [Ts*1e-8 0 ; 0 Ts];
Cc = [1 0];
sigmaw2 = 1*1*1e-1;


% Modello complessivo
sigmav2 = sigman2; % rumore di misura = rumore di quantizzazione
A = [Ad -Bd*Cc;
    zeros(size(Ac,1),size(Ad,2)) Ac];
B = [Bd;zeros(size(Ac,1),1)];

Bn = [zeros(size(Ad,1),size(Bc,2)); Bc];
%Bn = [[ 0.001*Ts*eye(size(Ad,1),size(Ad,1)); zeros(size(Ac,1),size(Ad,2))] Bn];

C = [Cd zeros(1,size(Ac,2))];
    
Q = Bn*Bn'*sigmaw2;
R = sigmav2;
S = zeros(size(A,1),size(R,2));
Qt = Q-S*inv(R)*S';
F = A-S*inv(R)*C;

%itera le equazioni del filtro di Kalman
xp(:,1) = zeros(size(A,1),1);
P = 1e-3*diag([1e3 1e3 1e3 1e3]);
%*eye(size(A,1));
urp(:,1) = [0 0 Cc]*xp(:,1);

for i=1:length(t)
[P,xf(:,i)]=update_u(yq(i),P,xp(:,i),F,C,R,0,u(:,i));
[P,xp(:,i+1)]=stepahead_u(yq(i),P,xf(:,i),F,R,Qt,S,B,u(:,i));
trace(P),i

urp(:,i+1) = [0 0 Cc]*xp(:,i+1);
urf(:,i) = [0 0 Cc]*xf(:,i);
yp(:,i+1) = C*xp(:,i+1);
yf(:,i) = C*xf(:,i);
end;

figure
plot(t,ur)
hold on
plot(t,urp(1:end-1),':r','linewidth',2);
plot(t,urf(1:end),':g','linewidth',2);
legend('coppia resistente','predizione coppia resistente','filtraggio coppia resistente')


figure
plot(t,y)
hold on
plot(t,yq,':','linewidth',2)
plot(t,yp(1:end-1),':r','linewidth',2);
plot(t,yf(1:end),':g','linewidth',2);
legend('posizione albero','misura posizione albero','predizione posizione albero','filtraggio posizione albero')


